'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _defineProperty2 = require('babel-runtime/core-js/object/define-property');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _defineProperty4 = require('babel-runtime/helpers/defineProperty');

var _defineProperty5 = _interopRequireDefault(_defineProperty4);

var _symbol = require('babel-runtime/core-js/symbol');

var _symbol2 = _interopRequireDefault(_symbol);

var _graphCrdt = require('graph-crdt');

var _pipeline = require('../pipeline');

var pipeline = _interopRequireWildcard(_pipeline);

var _mergeConfigs = require('../merge-configs');

var _mergeConfigs2 = _interopRequireDefault(_mergeConfigs);

var _context5 = require('./context');

var _context6 = _interopRequireDefault(_context5);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var settings = (0, _symbol2.default)('database configuration');

/**
 * Determines whether a connection is offline.
 * @param  {Connection} connection - Valid member of a connection group.
 * @return {Boolean} - Whether the connection is offline.
 */
var isOffline = function isOffline(connection) {
  return Boolean(connection.offline);
};

/**
 * Turns a node into a context.
 * @param  {Database} database - The current graph context.
 * @param  {Object|Node} data - Any data that can be transformed to a node.
 * @return {Context} - A new database context.
 */
var createContextFromNode = function createContextFromNode(database) {
  return function (data) {

    // Ensure the data is a node.
    var node = data instanceof _graphCrdt.Node ? data : _graphCrdt.Node.source(data);

    // Create a context from it.

    var _node$meta = node.meta(),
        uid = _node$meta.uid;

    var context = new _context6.default(database, { uid: uid });
    context.merge(node);

    return context;
  };
};

/**
 * Caches a node context in the database.
 * @param  {Database} graph - A database.
 * @param  {Node|Context} [node] - The data to cache.
 * @return {void}
 */
var cacheContext = function cacheContext(graph) {
  return function (context) {
    if (!context) {
      return;
    }

    var _context$meta = context.meta(),
        uid = _context$meta.uid;

    graph.merge((0, _defineProperty5.default)({}, uid, context));
  };
};

/**
 * Plugin manager for graph-crdt.
 * @class Database
 */

var Database = function (_Graph) {
  (0, _inherits3.default)(Database, _Graph);

  /**
   * Instantiates a new graph database.
   * @param  {Object} config - A database configuration object.
   */
  function Database(config) {
    (0, _classCallCheck3.default)(this, Database);

    var _this = (0, _possibleConstructorReturn3.default)(this, (Database.__proto__ || (0, _getPrototypeOf2.default)(Database)).call(this));

    _this[settings] = config;

    var createRouter = config.router;

    var router = createRouter ? createRouter(_this, config) : null;

    Object.defineProperty(_this, 'router', { value: router });

    var extensions = config.extend.root;

    /** Add API extensions from the config. */
    (0, _keys2.default)(extensions).forEach(function (key) {
      var value = extensions[key];

      /** Non-enumerable and immutable. */
      (0, _defineProperty3.default)(_this, key, { value: value });
    });
    return _this;
  }

  /**
   * Creates an in-memory database to aggregate changes before committing.
   * Storage and network plugins are excluded from the branch.
   * @return {Database} - A new, ephemeral database instance.
   */


  (0, _createClass3.default)(Database, [{
    key: 'branch',
    value: function branch() {
      var _settings = this[settings],
          extend = _settings.extend,
          hooks = _settings.hooks;


      var config = (0, _mergeConfigs2.default)([{ extend: extend, hooks: hooks }]);
      var branch = new Database(config);

      branch.merge(this);

      return branch;
    }

    /**
     * Applies a collection of changes all at once.
     * @param  {Graph|Database} update - A collection of updates.
     * @param  {Object} [options] - Override global configuration.
     * @return {Promise} - Resolves when the commit has been processed.
     */

  }, {
    key: 'commit',
    value: function () {
      var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(update) {
        var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

        var graph, contexts, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, _step$value, id, node, context, current, _current$meta, uid, config, deltas, write, writes;

        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:

                // Storage drivers get the full state of each node.
                graph = new _graphCrdt.Graph();
                contexts = this.new();
                _iteratorNormalCompletion = true;
                _didIteratorError = false;
                _iteratorError = undefined;
                _context.prev = 5;


                for (_iterator = (0, _getIterator3.default)(update); !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                  _step$value = (0, _slicedToArray3.default)(_step.value, 2), id = _step$value[0], node = _step$value[1];


                  if (node instanceof _context6.default) {
                    contexts.merge((0, _defineProperty5.default)({}, id, node));
                  } else {
                    context = new _context6.default(this, { uid: id });

                    context.merge(node);
                    contexts.merge((0, _defineProperty5.default)({}, id, context));
                  }

                  // If the node exists in the graph...
                  current = this.value(id);


                  if (current) {
                    _current$meta = current.meta(), uid = _current$meta.uid;

                    // Merge it with the update.

                    graph.merge((0, _defineProperty5.default)({}, uid, current));
                  }
                }

                _context.next = 13;
                break;

              case 9:
                _context.prev = 9;
                _context.t0 = _context['catch'](5);
                _didIteratorError = true;
                _iteratorError = _context.t0;

              case 13:
                _context.prev = 13;
                _context.prev = 14;

                if (!_iteratorNormalCompletion && _iterator.return) {
                  _iterator.return();
                }

              case 16:
                _context.prev = 16;

                if (!_didIteratorError) {
                  _context.next = 19;
                  break;
                }

                throw _iteratorError;

              case 19:
                return _context.finish(16);

              case 20:
                return _context.finish(13);

              case 21:
                graph.merge(update);

                _context.next = 24;
                return pipeline.before.write(this[settings], (0, _extends3.default)({
                  offline: this[settings].network.filter(isOffline)
                }, options, {
                  update: update,
                  graph: graph
                }));

              case 24:
                config = _context.sent;


                // Merge in the update.
                deltas = this.merge(contexts);
                write = (0, _extends3.default)({}, config, deltas);

                // Persist.

                writes = [];


                if (config.storage) {
                  writes.push(config.storage.write(write));
                }

                if (this.router) {
                  this.router.push(write);
                }

                // Wait for writes to finish.
                _context.next = 32;
                return _promise2.default.all(writes);

              case 32:
                _context.next = 34;
                return pipeline.after.write(this[settings], write);

              case 34:
                return _context.abrupt('return', _context.sent);

              case 35:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this, [[5, 9, 13, 21], [14,, 16, 20]]);
      }));

      function commit(_x) {
        return _ref.apply(this, arguments);
      }

      return commit;
    }()

    /**
     * Merges a node into the graph.
     * @param  {String} uid - The unique ID of the node.
     * @param  {Object} value - The fields to add/update.
     * @param  {Object} [options] - Override default behavior.
     * @return {Context} - Resolves to the context written.
     */

  }, {
    key: 'write',
    value: function () {
      var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(uid, value) {
        var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
        var context, current, field, update;
        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:

                // Turn the object into a database context.
                context = new _context6.default(this, { uid: uid });
                current = this.value(uid);
                _context2.t0 = _regenerator2.default.keys(value);

              case 3:
                if ((_context2.t1 = _context2.t0()).done) {
                  _context2.next = 11;
                  break;
                }

                field = _context2.t1.value;

                if (value.hasOwnProperty(field)) {
                  _context2.next = 7;
                  break;
                }

                return _context2.abrupt('continue', 3);

              case 7:

                // Turn nested nodes into edges.
                if (value[field] instanceof _graphCrdt.Node) {
                  value[field] = { edge: String(value[field]) };
                }

                // Ensure object updates increment state.
                if (current) {
                  context.merge((0, _defineProperty5.default)({}, field, current.value(field)));
                  context.meta(field).state = current.state(field);
                }
                _context2.next = 3;
                break;

              case 11:

                context.merge(value);

                update = _graphCrdt.Graph.source((0, _defineProperty5.default)({}, uid, context));
                _context2.next = 15;
                return this.commit(update, options);

              case 15:
                return _context2.abrupt('return', this.value(uid));

              case 16:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function write(_x3, _x4) {
        return _ref2.apply(this, arguments);
      }

      return write;
    }()

    /**
     * Reads a list of nodes.
     * @param  {String[]} keys - Keys to read.
     * @param  {Object} [options] - Read-level settings.
     * @return {Promise<Array<Context|null>>} - All the nodes it found.
     * @example
     * const [timeline, profile] = await db.nodes(['timeline', 'profile'])
     */

  }, {
    key: 'nodes',
    value: function () {
      var _ref3 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3(keys) {
        var _this2 = this;

        var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

        var config, getFinalValue, absentFromCache, reads, _result, readAction, read, _read, results, requestedKeys, isRelevantContext, result;

        return _regenerator2.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                _context3.next = 2;
                return pipeline.before.read.nodes(this[settings], (0, _extends3.default)({}, options, {
                  offline: this[settings].network.filter(isOffline),
                  keys: keys
                }));

              case 2:
                config = _context3.sent;

                // Load nodes from the cache and resolve with the value.
                getFinalValue = function getFinalValue(action) {
                  var nodes = config.keys.map(function (key) {
                    return _this2.value(key);
                  });

                  return pipeline.after.read.nodes(_this2[settings], (0, _extends3.default)({}, action, {
                    contexts: nodes
                  }));
                };

                // Find all the keys that aren't cached.


                absentFromCache = config.keys.filter(function (key) {
                  return !_this2.value(key);
                });
                reads = [];

                // Terminate early if everything is already cached.

                if (!(!absentFromCache.length && !config.force)) {
                  _context3.next = 11;
                  break;
                }

                _context3.next = 9;
                return getFinalValue(config);

              case 9:
                _result = _context3.sent;
                return _context3.abrupt('return', _result.contexts);

              case 11:

                // Only request missing data.
                readAction = (0, _extends3.default)({}, config, {
                  keys: absentFromCache
                });

                // Storage

                if (config.storage) {
                  read = config.storage.read(readAction);

                  reads.push(read);
                }

                // Network
                if (this.router) {
                  _read = this.router.pull(readAction);

                  reads.push(_read);
                }

                _context3.next = 16;
                return _promise2.default.all(reads);

              case 16:
                results = _context3.sent;
                requestedKeys = absentFromCache.reduce(function (keys, key) {
                  keys[key] = true;
                  return keys;
                }, {});

                // Ignore nodes we didn't request.

                isRelevantContext = function isRelevantContext(context) {
                  var _context$meta2 = context.meta(),
                      uid = _context$meta2.uid;

                  return requestedKeys.hasOwnProperty(uid);
                };

                // Process each node.


                results.forEach(function () {
                  var nodes = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
                  return nodes.filter(Boolean).map(createContextFromNode(_this2)).filter(isRelevantContext).forEach(cacheContext(_this2));
                });

                // Run the results through the pipeline.
                _context3.next = 22;
                return getFinalValue(readAction);

              case 22:
                result = _context3.sent;
                return _context3.abrupt('return', result.contexts);

              case 24:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function nodes(_x6) {
        return _ref3.apply(this, arguments);
      }

      return nodes;
    }()

    /**
     * Read a context from the database.
     * @param  {String} key - The node's unique ID.
     * @param  {Object} [options] - Plugin-level options.
     * @param  {ConnectionGroup|null} [options.network]
     * Override the group of network connections.
     * @param  {Object|null} [options.storage] - Storage plugins to use.
     * @param  {Boolean} [options.force] - Ignore cache and force read.
     * @return {Context|null} - Resolves to the node.
     */

  }, {
    key: 'read',
    value: function () {
      var _ref4 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee4(key) {
        var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

        var _ref5, _ref6, node;

        return _regenerator2.default.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                _context4.next = 2;
                return this.nodes([key], options);

              case 2:
                _ref5 = _context4.sent;
                _ref6 = (0, _slicedToArray3.default)(_ref5, 1);
                node = _ref6[0];
                return _context4.abrupt('return', node);

              case 6:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));

      function read(_x9) {
        return _ref4.apply(this, arguments);
      }

      return read;
    }()

    /**
     * description
     * @param  {Mixed} query - query that is passed to the engine
     * @param  {Object} options - options for executing the query
     * @return {undefined}
     */

  }, {
    key: 'query',
    value: function query(_query, options) {

      var config = this[settings];
      var engines = config.engines;
      var engine = options.engine;

      // Make sure the engine exists
      if (!engines.hasOwnProperty(engine)) {
        throw new Error('No engine is defined in options');
      }

      var queryEngine = engines[engine];
      return queryEngine.executeQuery(_query, this);
    }
  }]);
  return Database;
}(_graphCrdt.Graph);

/**
 * Creates a new instance of `Database` with or without `new`.
 * @param  {...Object} configs - Plugins and configurations for the database.
 * @return {Database} - A new database instance.
 */


function database() {
  for (var _len = arguments.length, configs = Array(_len), _key = 0; _key < _len; _key++) {
    configs[_key] = arguments[_key];
  }

  var config = (0, _mergeConfigs2.default)(configs);

  return new Database(config);
}

database.configuration = settings;

exports.default = database;