'use strict';

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _expect = require('expect');

var _expect2 = _interopRequireDefault(_expect);

var _root = require('../root');

var _root2 = _interopRequireDefault(_root);

var _context11 = require('../context');

var _context12 = _interopRequireDefault(_context11);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('A context', function () {
  var node = void 0,
      root = void 0;

  beforeEach(function () {
    root = (0, _root2.default)();

    var context = new _context12.default(root);

    var _context$meta = context.meta(),
        uid = _context$meta.uid;

    root.merge((0, _defineProperty3.default)({}, uid, context));
    node = root.value(uid);
  });

  it('should expose the database root', function () {
    (0, _expect2.default)(node.root).toBe(root);
  });

  it('should accept node constructor settings', function () {
    node = new _context12.default(root, {
      uid: 'potatoes'
    });

    var _node$meta = node.meta(),
        uid = _node$meta.uid;

    (0, _expect2.default)(uid).toBe('potatoes');
  });

  describe('fields()', function () {
    it('returns a promise', function () {
      var result = node.fields([]);

      (0, _expect2.default)(result).toBeA(_promise2.default);
    });

    it('resolves with an array', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee() {
      var result;
      return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              _context.next = 2;
              return node.fields([]);

            case 2:
              result = _context.sent;


              (0, _expect2.default)(result).toBeAn(Array);

            case 4:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, undefined);
    })));

    it('contains all the primitive fields', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2() {
      var result;
      return _regenerator2.default.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              node.merge({ one: true, two: 'also true' });
              _context2.next = 3;
              return node.fields(['two', 'one']);

            case 3:
              result = _context2.sent;


              (0, _expect2.default)(result).toEqual(['also true', true]);

            case 5:
            case 'end':
              return _context2.stop();
          }
        }
      }, _callee2, undefined);
    })));

    it('resolves pointers', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3() {
      var fields;
      return _regenerator2.default.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              _context3.next = 2;
              return node.write('self', node);

            case 2:
              _context3.next = 4;
              return node.fields(['self']);

            case 4:
              fields = _context3.sent;


              (0, _expect2.default)(fields).toEqual([node]);

            case 6:
            case 'end':
              return _context3.stop();
          }
        }
      }, _callee3, undefined);
    })));

    it('keeps nodes and primitives in the correct order', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee4() {
      var result;
      return _regenerator2.default.wrap(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              node.merge({ primitive: true, another: 'yep' });
              _context4.next = 3;
              return node.write('self', node);

            case 3:
              _context4.next = 5;
              return node.fields(['primitive', 'self', 'another']);

            case 5:
              result = _context4.sent;


              (0, _expect2.default)(result).toEqual([true, node, 'yep']);

            case 7:
            case 'end':
              return _context4.stop();
          }
        }
      }, _callee4, undefined);
    })));

    it('passes options to `nodes()`', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee5() {
      var options;
      return _regenerator2.default.wrap(function _callee5$(_context5) {
        while (1) {
          switch (_context5.prev = _context5.next) {
            case 0:
              (0, _expect.spyOn)(root, 'nodes').andCallThrough();

              _context5.next = 3;
              return node.write('self', node);

            case 3:
              _context5.next = 5;
              return node.fields(['self'], { storage: null });

            case 5:

              (0, _expect2.default)(root.nodes).toHaveBeenCalled();
              options = root.nodes.calls[0].arguments[1];

              (0, _expect2.default)(options).toExist();
              (0, _expect2.default)(options.storage).toBe(null);

            case 9:
            case 'end':
              return _context5.stop();
          }
        }
      }, _callee5, undefined);
    })));
  });

  describe('read()', function () {
    it('should return undefined if not found', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee6() {
      var result;
      return _regenerator2.default.wrap(function _callee6$(_context6) {
        while (1) {
          switch (_context6.prev = _context6.next) {
            case 0:
              _context6.next = 2;
              return node.read('nothing here');

            case 2:
              result = _context6.sent;

              (0, _expect2.default)(result).toBe(undefined);

            case 4:
            case 'end':
              return _context6.stop();
          }
        }
      }, _callee6, undefined);
    })));

    it('should return the value if it exists', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee7() {
      var enabled;
      return _regenerator2.default.wrap(function _callee7$(_context7) {
        while (1) {
          switch (_context7.prev = _context7.next) {
            case 0:
              _context7.next = 2;
              return node.write('enabled', true);

            case 2:
              _context7.next = 4;
              return node.read('enabled');

            case 4:
              enabled = _context7.sent;


              (0, _expect2.default)(enabled).toBe(true);

            case 6:
            case 'end':
              return _context7.stop();
          }
        }
      }, _callee7, undefined);
    })));

    it('should resolve contexts through the root', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee8() {
      var settings, read, result;
      return _regenerator2.default.wrap(function _callee8$(_context8) {
        while (1) {
          switch (_context8.prev = _context8.next) {
            case 0:
              settings = new _context12.default(root, { uid: 'settings' });
              _context8.next = 3;
              return node.write('settings', settings);

            case 3:
              read = (0, _expect.spyOn)(root, 'nodes');

              read.andReturn(_promise2.default.resolve([settings]));

              _context8.next = 7;
              return node.read('settings');

            case 7:
              result = _context8.sent;

              (0, _expect2.default)(result).toBe(settings);

            case 9:
            case 'end':
              return _context8.stop();
          }
        }
      }, _callee8, undefined);
    })));
  });

  describe('write()', function () {
    it('should write as an edge when given a context', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee9() {
      var settings, _node$meta2, value;

      return _regenerator2.default.wrap(function _callee9$(_context9) {
        while (1) {
          switch (_context9.prev = _context9.next) {
            case 0:
              settings = new _context12.default(root, { uid: 'settings' });
              _context9.next = 3;
              return node.write('settings', settings);

            case 3:
              _node$meta2 = node.meta('settings'), value = _node$meta2.value;

              (0, _expect2.default)(value).toEqual({ edge: 'settings' });

            case 5:
            case 'end':
              return _context9.stop();
          }
        }
      }, _callee9, undefined);
    })));

    it('increments node state', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee10() {
      return _regenerator2.default.wrap(function _callee10$(_context10) {
        while (1) {
          switch (_context10.prev = _context10.next) {
            case 0:
              _context10.next = 2;
              return node.write('count', 5);

            case 2:
              (0, _expect2.default)(node.value('count')).toBe(5);

              _context10.next = 5;
              return node.write('count', 7);

            case 5:
              (0, _expect2.default)(node.value('count')).toBe(7);

              _context10.next = 8;
              return node.write('count', 3);

            case 8:
              (0, _expect2.default)(node.value('count')).toBe(3);

              (0, _expect2.default)(node.state('count')).toBe(3);

            case 10:
            case 'end':
              return _context10.stop();
          }
        }
      }, _callee10, undefined);
    })));
  });

  describe('new()', function () {
    it('should return a new context', function () {
      var copy = node.new();
      (0, _expect2.default)(copy).toBeA(_context12.default);
    });

    it('should use the same uid', function () {
      var copy = node.new();
      (0, _expect2.default)(copy.meta()).toContain({
        uid: String(node)
      });
    });

    it('should attach the same root reference', function () {
      var copy = node.new();
      (0, _expect2.default)(copy.root).toBe(root);
    });

    it('should include API extensions', function () {
      var root = (0, _root2.default)({
        extend: {
          context: { hello: 'coder' }
        }
      });

      var ctx = new _context12.default(root);
      var copy = ctx.new();
      (0, _expect2.default)(copy.hello).toBe('coder');
    });
  });

  describe('API extension', function () {
    var extend = function extend(context) {
      return (0, _root2.default)({
        extend: { context: context }
      });
    };

    it('should contain extensions from the config', function () {
      var root = extend({
        wazzup: function wazzup() {}
      });

      var ctx = new _context12.default(root);
      (0, _expect2.default)(ctx.wazzup).toBeA(Function);
    });

    it('should be non-enumerable', function () {
      var root = extend({ 'non-enumerable': true });

      var ctx = new _context12.default(root);
      var keys = (0, _keys2.default)(ctx);
      (0, _expect2.default)(keys).toNotContain('non-enumerable');
    });

    it('should be immutable', function () {
      var root = extend({ hey: 'there' });

      var ctx = new _context12.default(root);
      (0, _expect2.default)(function () {
        ctx.hey = 'change!';
      }).toThrow(Error);
    });
  });
});