'use strict';

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _expect = require('expect');

var _expect2 = _interopRequireDefault(_expect);

var _graphCrdt = require('graph-crdt');

var _mocks = require('../../mocks');

var _connectionGroup = require('../../connection-group');

var _connectionGroup2 = _interopRequireDefault(_connectionGroup);

var _context53 = require('../context');

var _context54 = _interopRequireDefault(_context53);

var _root = require('../root');

var _root2 = _interopRequireDefault(_root);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('Database', function () {
  var settings = _root2.default.configuration;
  var db = void 0;

  beforeEach(function () {
    db = (0, _root2.default)();
  });

  it('is an object', function () {
    (0, _expect2.default)(db).toBeAn(Object);
  });

  it('generates a database configuration', function () {
    var config = db[settings];
    (0, _expect2.default)(config).toBeAn(Object);
  });

  it('creates a new graph', function () {
    (0, _expect2.default)(db).toBeA(_graphCrdt.Graph);
  });

  it('instantiates a new router', function () {
    var db = (0, _root2.default)({
      router: _mocks.createRouter
    });

    (0, _expect2.default)(db.router).toBeA(_mocks.Router);

    (0, _expect2.default)(_mocks.createRouter).toHaveBeenCalled();
    (0, _expect2.default)(_mocks.createRouter).toHaveBeenCalledWith(db, db[settings]);
  });

  describe('read()', function () {
    var router = void 0;

    beforeEach(function () {
      router = (0, _mocks.createRouter)();

      db = (0, _root2.default)({
        router: (0, _expect.createSpy)().andReturn(router)
      });
    });

    it('returns null if nothing is found', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee() {
      var settings;
      return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              _context.next = 2;
              return db.read('settings');

            case 2:
              settings = _context.sent;

              (0, _expect2.default)(settings).toBe(null);

            case 4:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, undefined);
    })));

    it('calls the router', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2() {
      var _router$pull$calls$0$, read;

      return _regenerator2.default.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              _context2.next = 2;
              return db.read('lobby');

            case 2:

              (0, _expect2.default)(router.pull).toHaveBeenCalled();
              _router$pull$calls$0$ = (0, _slicedToArray3.default)(router.pull.calls[0].arguments, 1), read = _router$pull$calls$0$[0];


              (0, _expect2.default)(read).toBeAn(Object);
              (0, _expect2.default)(read.keys).toEqual(['lobby']);

            case 6:
            case 'end':
              return _context2.stop();
          }
        }
      }, _callee2, undefined);
    })));

    it('resolves with the router response', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3() {
      var result;
      return _regenerator2.default.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              router.pull.andCall(function () {
                var node = new _graphCrdt.Node({ uid: 'weather' });
                node.merge({ response: 'router' });

                return [node.toJSON()];
              });

              _context3.next = 3;
              return db.read('weather');

            case 3:
              result = _context3.sent;

              (0, _expect2.default)(result).toBeA(_context54.default);
              (0, _expect2.default)(result.value('response')).toBe('router');

            case 6:
            case 'end':
              return _context3.stop();
          }
        }
      }, _callee3, undefined);
    })));

    it('ignores the cache when a flag is set', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee4() {
      return _regenerator2.default.wrap(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              _context4.next = 2;
              return db.write('game', { created: 1492219766210 });

            case 2:
              _context4.next = 4;
              return db.read('game');

            case 4:
              (0, _expect2.default)(router.pull).toNotHaveBeenCalled();

              _context4.next = 7;
              return db.read('game', { force: true });

            case 7:
              (0, _expect2.default)(router.pull).toHaveBeenCalled();

            case 8:
            case 'end':
              return _context4.stop();
          }
        }
      }, _callee4, undefined);
    })));

    describe('offline group', function () {
      it('is a connection group', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee5() {
        var _router$pull$calls$0$2, config;

        return _regenerator2.default.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                _context5.next = 2;
                return db.read('potato');

              case 2:

                (0, _expect2.default)(router.pull).toHaveBeenCalled();
                _router$pull$calls$0$2 = (0, _slicedToArray3.default)(router.pull.calls[0].arguments, 1), config = _router$pull$calls$0$2[0];


                (0, _expect2.default)(config.offline).toBeA(_connectionGroup2.default);
                (0, _expect2.default)([].concat((0, _toConsumableArray3.default)(config.offline))).toEqual([]);

              case 6:
              case 'end':
                return _context5.stop();
            }
          }
        }, _callee5, undefined);
      })));

      it('contains every offline item', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee6() {
        var conn1, conn2, _router$pull$calls$0$3, config;

        return _regenerator2.default.wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                conn1 = new _mocks.Connection({ id: 'conn1' });
                conn2 = new _mocks.Connection({ id: 'conn2' });


                conn1.offline = true;
                conn2.offline = false;

                db[_root2.default.configuration].network.add(conn1);
                db[_root2.default.configuration].network.add(conn2);

                _context6.next = 8;
                return db.read('things');

              case 8:
                _router$pull$calls$0$3 = (0, _slicedToArray3.default)(router.pull.calls[0].arguments, 1), config = _router$pull$calls$0$3[0];

                (0, _expect2.default)([].concat((0, _toConsumableArray3.default)(config.offline))).toEqual([conn1]);

              case 10:
              case 'end':
                return _context6.stop();
            }
          }
        }, _callee6, undefined);
      })));
    });
  });

  describe('nodes()', function () {
    var storage = void 0;

    beforeEach(function () {
      storage = new _mocks.Storage();
      storage.read = (0, _expect.createSpy)();

      db = (0, _root2.default)({ storage: storage, router: _mocks.createRouter });

      db.router.pull = (0, _expect.createSpy)();
    });

    it('is a function', function () {
      (0, _expect2.default)(db.nodes).toBeA(Function);
    });

    it('returns a promise', function () {
      var value = db.nodes(['stuff']);

      (0, _expect2.default)(value).toBeA(_promise2.default);
    });

    it('resolves with an array', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee7() {
      var value;
      return _regenerator2.default.wrap(function _callee7$(_context7) {
        while (1) {
          switch (_context7.prev = _context7.next) {
            case 0:
              _context7.next = 2;
              return db.nodes([]);

            case 2:
              value = _context7.sent;


              (0, _expect2.default)(value).toEqual([]);

            case 4:
            case 'end':
              return _context7.stop();
          }
        }
      }, _callee7, undefined);
    })));

    it('ignores storage if the option is disabled', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee8() {
      return _regenerator2.default.wrap(function _callee8$(_context8) {
        while (1) {
          switch (_context8.prev = _context8.next) {
            case 0:
              _context8.next = 2;
              return db.nodes(['potato'], { storage: null });

            case 2:

              (0, _expect2.default)(storage.read).toNotHaveBeenCalled();

            case 3:
            case 'end':
              return _context8.stop();
          }
        }
      }, _callee8, undefined);
    })));

    it('creates a read action', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee9() {
      var keys, _storage$read$calls$, action;

      return _regenerator2.default.wrap(function _callee9$(_context9) {
        while (1) {
          switch (_context9.prev = _context9.next) {
            case 0:
              keys = ['key1', 'key2'];
              _context9.next = 3;
              return db.nodes(keys);

            case 3:

              (0, _expect2.default)(storage.read).toHaveBeenCalled();
              _storage$read$calls$ = (0, _slicedToArray3.default)(storage.read.calls[0].arguments, 1), action = _storage$read$calls$[0];


              (0, _expect2.default)(action.keys).toEqual(keys);

            case 6:
            case 'end':
              return _context9.stop();
          }
        }
      }, _callee9, undefined);
    })));

    it('returns the nodes given by storage', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee11() {
      var nodes, _nodes, value;

      return _regenerator2.default.wrap(function _callee11$(_context11) {
        while (1) {
          switch (_context11.prev = _context11.next) {
            case 0:
              storage.read.andCall((0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee10() {
                var node;
                return _regenerator2.default.wrap(function _callee10$(_context10) {
                  while (1) {
                    switch (_context10.prev = _context10.next) {
                      case 0:
                        node = new _graphCrdt.Node({ uid: 'a-node' });

                        node.merge({ read: true });

                        return _context10.abrupt('return', [node]);

                      case 3:
                      case 'end':
                        return _context10.stop();
                    }
                  }
                }, _callee10, undefined);
              })));

              _context11.next = 3;
              return db.nodes(['a-node']);

            case 3:
              nodes = _context11.sent;
              _nodes = (0, _slicedToArray3.default)(nodes, 1), value = _nodes[0];


              (0, _expect2.default)(nodes.length).toBe(1);
              (0, _expect2.default)(value).toBeA(_context54.default);
              (0, _expect2.default)(value.snapshot()).toEqual({ read: true });

            case 8:
            case 'end':
              return _context11.stop();
          }
        }
      }, _callee11, undefined);
    })));

    it('uses the correct context ID for resolved nodes', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee13() {
      var uid, _ref14, _ref15, result;

      return _regenerator2.default.wrap(function _callee13$(_context13) {
        while (1) {
          switch (_context13.prev = _context13.next) {
            case 0:
              uid = 'result';


              storage.read.andCall((0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee12() {
                var node;
                return _regenerator2.default.wrap(function _callee12$(_context12) {
                  while (1) {
                    switch (_context12.prev = _context12.next) {
                      case 0:
                        node = new _graphCrdt.Node({ uid: uid });

                        node.merge({ resolved: true });

                        return _context12.abrupt('return', [node.toJSON()]);

                      case 3:
                      case 'end':
                        return _context12.stop();
                    }
                  }
                }, _callee12, undefined);
              })));

              _context13.next = 4;
              return db.nodes([uid]);

            case 4:
              _ref14 = _context13.sent;
              _ref15 = (0, _slicedToArray3.default)(_ref14, 1);
              result = _ref15[0];


              (0, _expect2.default)(result.meta().uid).toBe(uid);

            case 8:
            case 'end':
              return _context13.stop();
          }
        }
      }, _callee13, undefined);
    })));

    it('returns `null` if no value is given', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee15() {
      var nodes;
      return _regenerator2.default.wrap(function _callee15$(_context15) {
        while (1) {
          switch (_context15.prev = _context15.next) {
            case 0:
              storage.read.andCall((0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee14() {
                return _regenerator2.default.wrap(function _callee14$(_context14) {
                  while (1) {
                    switch (_context14.prev = _context14.next) {
                      case 0:
                        return _context14.abrupt('return', [undefined]);

                      case 1:
                      case 'end':
                        return _context14.stop();
                    }
                  }
                }, _callee14, undefined);
              })));
              _context15.next = 3;
              return db.nodes(['stuff']);

            case 3:
              nodes = _context15.sent;


              (0, _expect2.default)(nodes).toEqual([null]);

            case 5:
            case 'end':
              return _context15.stop();
          }
        }
      }, _callee15, undefined);
    })));

    it('supports node resolve values', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee16() {
      var _ref19, _ref20, node;

      return _regenerator2.default.wrap(function _callee16$(_context16) {
        while (1) {
          switch (_context16.prev = _context16.next) {
            case 0:
              storage.read.andCall(function () {
                var node = new _graphCrdt.Node({ uid: 'result' });
                node.merge({ resolved: true });

                return [node];
              });

              _context16.next = 3;
              return db.nodes(['result']);

            case 3:
              _ref19 = _context16.sent;
              _ref20 = (0, _slicedToArray3.default)(_ref19, 1);
              node = _ref20[0];


              (0, _expect2.default)(node).toBeA(_context54.default);
              (0, _expect2.default)(node.meta().uid).toEqual('result');
              (0, _expect2.default)(node.snapshot()).toEqual({ resolved: true });

            case 9:
            case 'end':
              return _context16.stop();
          }
        }
      }, _callee16, undefined);
    })));

    it('caches the value', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee17() {
      var _ref22, _ref23, node;

      return _regenerator2.default.wrap(function _callee17$(_context17) {
        while (1) {
          switch (_context17.prev = _context17.next) {
            case 0:
              storage.read.andCall(function () {
                var node = new _graphCrdt.Node({ uid: 'stuff' });
                node.merge({ stuff: true });

                return [node];
              });

              _context17.next = 3;
              return db.nodes(['stuff']);

            case 3:
              _ref22 = _context17.sent;
              _ref23 = (0, _slicedToArray3.default)(_ref22, 1);
              node = _ref23[0];
              _context17.t0 = (0, _expect2.default)(node);
              _context17.next = 9;
              return db.read('stuff');

            case 9:
              _context17.t1 = _context17.sent;

              _context17.t0.toBe.call(_context17.t0, _context17.t1, 'Node was not cached.');

            case 11:
            case 'end':
              return _context17.stop();
          }
        }
      }, _callee17, undefined);
    })));

    it('returns the nodes in the order you requested', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee18() {
      var _ref25, _ref26, first, second;

      return _regenerator2.default.wrap(function _callee18$(_context18) {
        while (1) {
          switch (_context18.prev = _context18.next) {
            case 0:
              storage.read.andCall(function () {
                var nodes = [new _graphCrdt.Node({ uid: 'second' }), new _graphCrdt.Node({ uid: 'first' })];

                nodes.forEach(function (node) {
                  return node.merge((0, _defineProperty3.default)({}, node.meta().uid, true));
                });

                return nodes;
              });

              _context18.next = 3;
              return db.nodes(['first', 'second']);

            case 3:
              _ref25 = _context18.sent;
              _ref26 = (0, _slicedToArray3.default)(_ref25, 2);
              first = _ref26[0];
              second = _ref26[1];


              (0, _expect2.default)(first.meta().uid).toBe('first');
              (0, _expect2.default)(second.meta().uid).toBe('second');

            case 9:
            case 'end':
              return _context18.stop();
          }
        }
      }, _callee18, undefined);
    })));

    it('does not attempt to read the same nodes twice', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee19() {
      var _storage$read$calls$2, action1, _storage$read$calls$3, action2;

      return _regenerator2.default.wrap(function _callee19$(_context19) {
        while (1) {
          switch (_context19.prev = _context19.next) {
            case 0:
              storage.read.andCall(function () {
                var node = new _graphCrdt.Node({ uid: 'first' });
                node.merge({ first: true });

                return [node];
              });

              _context19.next = 3;
              return db.nodes(['first', 'second']);

            case 3:
              _context19.next = 5;
              return db.nodes(['first', 'second']);

            case 5:

              (0, _expect2.default)(storage.read.calls.length).toBe(2);
              _storage$read$calls$2 = (0, _slicedToArray3.default)(storage.read.calls[0].arguments, 1), action1 = _storage$read$calls$2[0];
              _storage$read$calls$3 = (0, _slicedToArray3.default)(storage.read.calls[1].arguments, 1), action2 = _storage$read$calls$3[0];


              (0, _expect2.default)(action1.keys).toEqual(['first', 'second']);
              (0, _expect2.default)(action2.keys).toEqual(['second']);

            case 10:
            case 'end':
              return _context19.stop();
          }
        }
      }, _callee19, undefined);
    })));

    it('does not read from storage if everything is cached', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee20() {
      return _regenerator2.default.wrap(function _callee20$(_context20) {
        while (1) {
          switch (_context20.prev = _context20.next) {
            case 0:
              storage.read.andCall(function () {
                return [new _graphCrdt.Node({ uid: 'first' }), new _graphCrdt.Node({ uid: 'second' })];
              });

              _context20.next = 3;
              return db.nodes(['first', 'second']);

            case 3:
              _context20.next = 5;
              return db.nodes(['first', 'second']);

            case 5:

              (0, _expect2.default)(storage.read.calls.length).toBe(1);

            case 6:
            case 'end':
              return _context20.stop();
          }
        }
      }, _callee20, undefined);
    })));

    it('reads from the network', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee21() {
      var keys, _db$router$pull$calls, action;

      return _regenerator2.default.wrap(function _callee21$(_context21) {
        while (1) {
          switch (_context21.prev = _context21.next) {
            case 0:
              keys = ['key1', 'key2'];
              _context21.next = 3;
              return db.nodes(keys);

            case 3:

              (0, _expect2.default)(db.router.pull).toHaveBeenCalled();
              _db$router$pull$calls = (0, _slicedToArray3.default)(db.router.pull.calls[0].arguments, 1), action = _db$router$pull$calls[0];


              (0, _expect2.default)(action).toBeAn(Object);
              (0, _expect2.default)(action.keys).toEqual(keys);

            case 7:
            case 'end':
              return _context21.stop();
          }
        }
      }, _callee21, undefined);
    })));

    it('uses the resolve value from the router', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee23() {
      var _ref32, _ref33, node;

      return _regenerator2.default.wrap(function _callee23$(_context23) {
        while (1) {
          switch (_context23.prev = _context23.next) {
            case 0:
              db.router.pull.andCall((0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee22() {
                var node;
                return _regenerator2.default.wrap(function _callee22$(_context22) {
                  while (1) {
                    switch (_context22.prev = _context22.next) {
                      case 0:
                        node = new _graphCrdt.Node({ uid: 'router' });

                        node.merge({ routed: true });

                        return _context22.abrupt('return', [node]);

                      case 3:
                      case 'end':
                        return _context22.stop();
                    }
                  }
                }, _callee22, undefined);
              })));

              _context23.next = 3;
              return db.nodes(['router']);

            case 3:
              _ref32 = _context23.sent;
              _ref33 = (0, _slicedToArray3.default)(_ref32, 1);
              node = _ref33[0];


              (0, _expect2.default)(node).toBeA(_context54.default);
              (0, _expect2.default)(node.snapshot()).toEqual({ routed: true });

            case 8:
            case 'end':
              return _context23.stop();
          }
        }
      }, _callee23, undefined);
    })));

    it('does not consult the router if everything is cached', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee24() {
      return _regenerator2.default.wrap(function _callee24$(_context24) {
        while (1) {
          switch (_context24.prev = _context24.next) {
            case 0:
              db.router.pull.andCall(function () {
                return [new _graphCrdt.Node({ uid: 'result' })];
              });

              _context24.next = 3;
              return db.nodes(['result']);

            case 3:
              _context24.next = 5;
              return db.nodes(['result']);

            case 5:

              // Cached the first time.
              (0, _expect2.default)(db.router.pull.calls.length).toBe(1);

            case 6:
            case 'end':
              return _context24.stop();
          }
        }
      }, _callee24, undefined);
    })));

    it('can override the network group', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee25() {
      var network, _db$router$pull$calls2, read;

      return _regenerator2.default.wrap(function _callee25$(_context25) {
        while (1) {
          switch (_context25.prev = _context25.next) {
            case 0:
              network = _connectionGroup2.default.ensure(new _mocks.Connection());
              _context25.next = 3;
              return db.nodes(['stuff'], { network: network });

            case 3:
              _db$router$pull$calls2 = (0, _slicedToArray3.default)(db.router.pull.calls[0].arguments, 1), read = _db$router$pull$calls2[0];

              (0, _expect2.default)([].concat((0, _toConsumableArray3.default)(read.network))).toEqual([].concat((0, _toConsumableArray3.default)(network)));

            case 5:
            case 'end':
              return _context25.stop();
          }
        }
      }, _callee25, undefined);
    })));

    it('invokes the pre-read pipeline', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee26() {
      var hook, db, _storage$read$calls$4, action;

      return _regenerator2.default.wrap(function _callee26$(_context26) {
        while (1) {
          switch (_context26.prev = _context26.next) {
            case 0:
              hook = (0, _expect.createSpy)().andCall(function (action) {
                return (0, _extends3.default)({}, action, {
                  keys: action.keys.map(function (key) {
                    return 'replaced/' + key;
                  })
                });
              });
              db = (0, _root2.default)({
                storage: storage,
                hooks: {
                  before: { read: { nodes: hook } }
                }
              });
              _context26.next = 4;
              return db.nodes(['stuff']);

            case 4:
              _storage$read$calls$4 = (0, _slicedToArray3.default)(storage.read.calls[0].arguments, 1), action = _storage$read$calls$4[0];

              (0, _expect2.default)(action.keys).toEqual(['replaced/stuff']);

            case 6:
            case 'end':
              return _context26.stop();
          }
        }
      }, _callee26, undefined);
    })));

    it('invokes the post-read pipeline', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee27() {
      var hook, db, nodes, _hook$calls$0$argumen, action;

      return _regenerator2.default.wrap(function _callee27$(_context27) {
        while (1) {
          switch (_context27.prev = _context27.next) {
            case 0:
              hook = (0, _expect.createSpy)().andCall(function (action) {
                return action;
              });

              storage.read.andCall(function () {
                return [new _graphCrdt.Node({ uid: 'things' })];
              });

              db = (0, _root2.default)({
                storage: storage,
                hooks: {
                  after: { read: { nodes: hook } }
                }
              });
              _context27.next = 5;
              return db.nodes(['things']);

            case 5:
              nodes = _context27.sent;


              (0, _expect2.default)(hook).toHaveBeenCalled();
              _hook$calls$0$argumen = (0, _slicedToArray3.default)(hook.calls[0].arguments, 1), action = _hook$calls$0$argumen[0];


              (0, _expect2.default)(action.contexts).toEqual(nodes);

            case 9:
            case 'end':
              return _context27.stop();
          }
        }
      }, _callee27, undefined);
    })));

    it('allows hooks to override the node values', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee28() {
      var hook, db, nodes;
      return _regenerator2.default.wrap(function _callee28$(_context28) {
        while (1) {
          switch (_context28.prev = _context28.next) {
            case 0:
              hook = (0, _expect.createSpy)().andCall(function (action) {
                return (0, _extends3.default)({}, action, {
                  contexts: ['ha!']
                });
              });
              db = (0, _root2.default)({
                hooks: {
                  after: { read: { nodes: hook } }
                }
              });
              _context28.next = 4;
              return db.nodes(['something']);

            case 4:
              nodes = _context28.sent;


              (0, _expect2.default)(nodes).toEqual(['ha!']);

            case 6:
            case 'end':
              return _context28.stop();
          }
        }
      }, _callee28, undefined);
    })));

    it('calls the post-read hooks for cached values', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee29() {
      var hook, db, nodes;
      return _regenerator2.default.wrap(function _callee29$(_context29) {
        while (1) {
          switch (_context29.prev = _context29.next) {
            case 0:
              hook = (0, _expect.createSpy)().andCall(function (action) {
                return (0, _extends3.default)({}, action, {
                  contexts: ['replaced']
                });
              });
              db = (0, _root2.default)({
                hooks: {
                  after: { read: { nodes: hook } }
                }
              });
              _context29.next = 4;
              return db.write('things', {});

            case 4:
              _context29.next = 6;
              return db.nodes(['things']);

            case 6:
              nodes = _context29.sent;


              (0, _expect2.default)(nodes).toEqual(['replaced']);

            case 8:
            case 'end':
              return _context29.stop();
          }
        }
      }, _callee29, undefined);
    })));

    it('reads anyway if the read is forced', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee30() {
      return _regenerator2.default.wrap(function _callee30$(_context30) {
        while (1) {
          switch (_context30.prev = _context30.next) {
            case 0:
              _context30.next = 2;
              return db.write('node', {});

            case 2:
              _context30.next = 4;
              return db.nodes(['node'], { force: true });

            case 4:

              (0, _expect2.default)(storage.read).toHaveBeenCalled();

            case 5:
            case 'end':
              return _context30.stop();
          }
        }
      }, _callee30, undefined);
    })));

    it('ignores nodes it did not ask for', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee32() {
      var _ref43, _ref44, node;

      return _regenerator2.default.wrap(function _callee32$(_context32) {
        while (1) {
          switch (_context32.prev = _context32.next) {
            case 0:
              storage.read.andCall((0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee31() {
                return _regenerator2.default.wrap(function _callee31$(_context31) {
                  while (1) {
                    switch (_context31.prev = _context31.next) {
                      case 0:
                        return _context31.abrupt('return', [new _graphCrdt.Node({ uid: 'ignore-me' })]);

                      case 1:
                      case 'end':
                        return _context31.stop();
                    }
                  }
                }, _callee31, undefined);
              })));

              _context32.next = 3;
              return db.nodes(['something-else']);

            case 3:
              _ref43 = _context32.sent;
              _ref44 = (0, _slicedToArray3.default)(_ref43, 1);
              node = _ref44[0];


              (0, _expect2.default)(node).toNotExist();
              (0, _expect2.default)(db.value('ignore-me')).toBe(null);

            case 8:
            case 'end':
              return _context32.stop();
          }
        }
      }, _callee32, undefined);
    })));
  });

  describe('branch()', function () {
    it('returns a new database', function () {
      var result = db.branch();

      (0, _expect2.default)(result).toBeA(_graphCrdt.Graph);
      (0, _expect2.default)(result).toNotBe(db);
    });

    it('contains a config', function () {
      var branch = db.branch();
      var config = branch[_root2.default.configuration];

      (0, _expect2.default)(config).toBeAn(Object);
    });

    it('includes all hooks', function () {
      var hooks = {
        before: { write: (0, _expect.createSpy)() },
        after: { write: (0, _expect.createSpy)() }
      };

      var db = (0, _root2.default)({ hooks: hooks });

      var branch = db.branch();
      var config = branch[_root2.default.configuration];

      (0, _expect2.default)(config.hooks.before).toContain({ write: [hooks.before.write] });
      (0, _expect2.default)(config.hooks.after).toContain({ write: [hooks.after.write] });
    });

    it('includes all api extensions', function () {
      var extend = {
        context: { method: (0, _expect.createSpy)() },
        root: { method: (0, _expect.createSpy)() }
      };

      var db = (0, _root2.default)({ extend: extend });

      var branch = db.branch();
      var config = branch[_root2.default.configuration];

      (0, _expect2.default)(config.extend.root).toEqual(extend.root);
      (0, _expect2.default)(config.extend.context).toEqual(extend.context);
    });

    it('does not contain storage plugins', function () {
      var db = (0, _root2.default)({ storage: new _mocks.Storage() });
      var branch = db.branch();

      var config = branch[_root2.default.configuration];

      (0, _expect2.default)(config.storage).toEqual(null);
    });

    it('does not contain network plugins', function () {
      var connection = new _mocks.Connection();
      var network = new _connectionGroup2.default();
      network.add(connection);

      var db = (0, _root2.default)({ network: network });
      var branch = db.branch();

      var config = branch[_root2.default.configuration];

      // Empty - no connection.
      (0, _expect2.default)(config.network).toBeA(_connectionGroup2.default);
      (0, _expect2.default)([].concat((0, _toConsumableArray3.default)(config.network))).toEqual([]);
    });

    it('does not contain a router', function () {
      var db = (0, _root2.default)({ router: _mocks.Router.create });
      var branch = db.branch();
      var config = branch[_root2.default.configuration];

      (0, _expect2.default)(config.router).toBe(null);
    });

    it('contains all the current data', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee33() {
      var branch, node;
      return _regenerator2.default.wrap(function _callee33$(_context33) {
        while (1) {
          switch (_context33.prev = _context33.next) {
            case 0:
              _context33.next = 2;
              return db.write('user', { name: 'Bob' });

            case 2:
              branch = db.branch();
              node = branch.value('user');


              (0, _expect2.default)(node).toBeA(_graphCrdt.Node);
              (0, _expect2.default)([].concat((0, _toConsumableArray3.default)(node))).toEqual([['name', 'Bob']]);

            case 6:
            case 'end':
              return _context33.stop();
          }
        }
      }, _callee33, undefined);
    })));
  });

  describe('commit()', function () {
    var graph = void 0,
        node = void 0,
        beforeWrite = void 0,
        afterWrite = void 0,
        storage = void 0,
        router = void 0;
    var Identity = function Identity(value) {
      return value;
    };

    beforeEach(function () {
      beforeWrite = (0, _expect.createSpy)().andCall(Identity);
      afterWrite = (0, _expect.createSpy)().andCall(Identity);
      graph = new _graphCrdt.Graph();
      node = new _graphCrdt.Node();

      storage = new _mocks.Storage();
      storage.write = (0, _expect.createSpy)();

      router = new _mocks.Router();

      graph.merge((0, _defineProperty3.default)({}, node, node));

      db = (0, _root2.default)({
        hooks: {
          before: {
            write: beforeWrite
          },
          after: {
            write: afterWrite
          }
        },

        storage: storage,

        router: (0, _expect.createSpy)().andReturn(router)
      });
    });

    it('returns a promise', function () {
      var result = db.commit(graph);

      (0, _expect2.default)(result).toBeA(_promise2.default);
    });

    it('triggers the before write pipeline', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee34() {
      var _beforeWrite$calls$0$, pipe;

      return _regenerator2.default.wrap(function _callee34$(_context34) {
        while (1) {
          switch (_context34.prev = _context34.next) {
            case 0:
              _context34.next = 2;
              return db.commit(graph);

            case 2:

              (0, _expect2.default)(beforeWrite).toHaveBeenCalled();

              _beforeWrite$calls$0$ = (0, _slicedToArray3.default)(beforeWrite.calls[0].arguments, 1), pipe = _beforeWrite$calls$0$[0];

              (0, _expect2.default)(pipe.update).toBeA(_graphCrdt.Graph);
              (0, _expect2.default)([].concat((0, _toConsumableArray3.default)(pipe.update))).toEqual([].concat((0, _toConsumableArray3.default)(graph)));

            case 6:
            case 'end':
              return _context34.stop();
          }
        }
      }, _callee34, undefined);
    })));

    it('calls storage drivers', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee35() {
      var _storage$write$calls$, write;

      return _regenerator2.default.wrap(function _callee35$(_context35) {
        while (1) {
          switch (_context35.prev = _context35.next) {
            case 0:
              _context35.next = 2;
              return db.commit(graph);

            case 2:

              (0, _expect2.default)(storage.write).toHaveBeenCalled();
              _storage$write$calls$ = (0, _slicedToArray3.default)(storage.write.calls[0].arguments, 1), write = _storage$write$calls$[0];


              (0, _expect2.default)(write.graph).toBeA(_graphCrdt.Graph);

            case 5:
            case 'end':
              return _context35.stop();
          }
        }
      }, _callee35, undefined);
    })));

    it('calls the router', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee36() {
      var _router$push$calls$0$, config;

      return _regenerator2.default.wrap(function _callee36$(_context36) {
        while (1) {
          switch (_context36.prev = _context36.next) {
            case 0:
              _context36.next = 2;
              return db.commit(graph);

            case 2:

              (0, _expect2.default)(router.push).toHaveBeenCalled();

              _router$push$calls$0$ = (0, _slicedToArray3.default)(router.push.calls[0].arguments, 1), config = _router$push$calls$0$[0];


              (0, _expect2.default)(config).toBeAn(Object);
              (0, _expect2.default)(config.update).toBeA(_graphCrdt.Graph);

            case 6:
            case 'end':
              return _context36.stop();
          }
        }
      }, _callee36, undefined);
    })));

    describe('offline group', function () {
      it('is a ConnectionGroup', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee37() {
        var _router$push$calls$0$2, config;

        return _regenerator2.default.wrap(function _callee37$(_context37) {
          while (1) {
            switch (_context37.prev = _context37.next) {
              case 0:
                _context37.next = 2;
                return db.commit(graph);

              case 2:

                (0, _expect2.default)(router.push).toHaveBeenCalled();
                _router$push$calls$0$2 = (0, _slicedToArray3.default)(router.push.calls[0].arguments, 1), config = _router$push$calls$0$2[0];


                (0, _expect2.default)(config.offline).toBeA(_connectionGroup2.default);

              case 5:
              case 'end':
                return _context37.stop();
            }
          }
        }, _callee37, undefined);
      })));

      it('it contains every offline connection', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee38() {
        var conn1, conn2, _router$push$calls$0$3, config;

        return _regenerator2.default.wrap(function _callee38$(_context38) {
          while (1) {
            switch (_context38.prev = _context38.next) {
              case 0:
                conn1 = new _mocks.Connection({ id: 'conn1' });
                conn2 = new _mocks.Connection({ id: 'conn2' });


                conn1.offline = true;
                conn2.offline = false;

                db[_root2.default.configuration].network.add(conn1);
                db[_root2.default.configuration].network.add(conn2);

                _context38.next = 8;
                return db.commit(graph);

              case 8:
                _router$push$calls$0$3 = (0, _slicedToArray3.default)(router.push.calls[0].arguments, 1), config = _router$push$calls$0$3[0];

                (0, _expect2.default)([].concat((0, _toConsumableArray3.default)(config.offline))).toEqual([conn1]);

              case 10:
              case 'end':
                return _context38.stop();
            }
          }
        }, _callee38, undefined);
      })));
    });

    it('passes the full state of each node to storage', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee39() {
      var update, node, _storage$write$calls$2, write, state;

      return _regenerator2.default.wrap(function _callee39$(_context39) {
        while (1) {
          switch (_context39.prev = _context39.next) {
            case 0:
              _context39.next = 2;
              return db.write('state', { existing: true });

            case 2:
              update = new _graphCrdt.Graph();
              node = new _graphCrdt.Node({ uid: 'state' });

              node.merge({ new: true });
              update.merge((0, _defineProperty3.default)({}, node, node));
              storage.write.reset();
              _context39.next = 9;
              return db.commit(update);

            case 9:

              (0, _expect2.default)(storage.write).toHaveBeenCalled();
              _storage$write$calls$2 = (0, _slicedToArray3.default)(storage.write.calls[0].arguments, 1), write = _storage$write$calls$2[0];
              state = write.graph.value('state');


              (0, _expect2.default)([].concat((0, _toConsumableArray3.default)(state))).toEqual([].concat((0, _toConsumableArray3.default)(_graphCrdt.Node.from({
                existing: true,
                new: true
              }))));

            case 13:
            case 'end':
              return _context39.stop();
          }
        }
      }, _callee39, undefined);
    })));

    it('only contains nodes which updated', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee40() {
      var update, _storage$write$calls$3, write;

      return _regenerator2.default.wrap(function _callee40$(_context40) {
        while (1) {
          switch (_context40.prev = _context40.next) {
            case 0:
              update = new _graphCrdt.Graph();
              _context40.next = 3;
              return db.write('unrelated', { ignore: 'please' });

            case 3:
              storage.write.reset();
              _context40.next = 6;
              return db.commit(update);

            case 6:
              _storage$write$calls$3 = (0, _slicedToArray3.default)(storage.write.calls[0].arguments, 1), write = _storage$write$calls$3[0];

              (0, _expect2.default)(write.graph.value('unrelated')).toNotExist();

            case 8:
            case 'end':
              return _context40.stop();
          }
        }
      }, _callee40, undefined);
    })));

    it('uses the storage options if given', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee41() {
      return _regenerator2.default.wrap(function _callee41$(_context41) {
        while (1) {
          switch (_context41.prev = _context41.next) {
            case 0:
              _context41.next = 2;
              return db.commit(graph, { storage: null });

            case 2:

              (0, _expect2.default)(storage.write).toNotHaveBeenCalled();

            case 3:
            case 'end':
              return _context41.stop();
          }
        }
      }, _callee41, undefined);
    })));

    it('passes the deltas to plugins', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee42() {
      var _router$push$calls$0$4, push, _storage$write$calls$4, write;

      return _regenerator2.default.wrap(function _callee42$(_context42) {
        while (1) {
          switch (_context42.prev = _context42.next) {
            case 0:
              _context42.next = 2;
              return db.commit(graph);

            case 2:
              _router$push$calls$0$4 = (0, _slicedToArray3.default)(router.push.calls[0].arguments, 1), push = _router$push$calls$0$4[0];

              (0, _expect2.default)(push.history).toBeA(_graphCrdt.Graph);
              (0, _expect2.default)(push.update).toBeA(_graphCrdt.Graph);

              _storage$write$calls$4 = (0, _slicedToArray3.default)(storage.write.calls[0].arguments, 1), write = _storage$write$calls$4[0];

              (0, _expect2.default)(write.history).toBeA(_graphCrdt.Graph);
              (0, _expect2.default)(write.update).toBeA(_graphCrdt.Graph);

            case 8:
            case 'end':
              return _context42.stop();
          }
        }
      }, _callee42, undefined);
    })));

    it('only sends the changes', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee43() {
      var weather, changes, _router$push$calls$0$5, push;

      return _regenerator2.default.wrap(function _callee43$(_context43) {
        while (1) {
          switch (_context43.prev = _context43.next) {
            case 0:
              weather = new _graphCrdt.Node({ uid: 'weather' });

              weather.merge({ condition: 'sunny', temp: 65 });
              db.merge((0, _defineProperty3.default)({}, weather, weather));

              changes = new _graphCrdt.Graph();

              changes.merge(db);
              changes.value(weather).merge({ temp: 70 });
              _context43.next = 8;
              return db.commit(changes);

            case 8:
              _router$push$calls$0$5 = (0, _slicedToArray3.default)(router.push.calls[0].arguments, 1), push = _router$push$calls$0$5[0];

              (0, _expect2.default)([].concat((0, _toConsumableArray3.default)(push.update)).length).toBe(1);
              (0, _expect2.default)([].concat((0, _toConsumableArray3.default)(push.update.value('weather')))).toEqual([['temp', 70]]);

            case 11:
            case 'end':
              return _context43.stop();
          }
        }
      }, _callee43, undefined);
    })));

    it('caches the value after pipeline validation', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee44() {
      var node, cached;
      return _regenerator2.default.wrap(function _callee44$(_context44) {
        while (1) {
          switch (_context44.prev = _context44.next) {
            case 0:
              node = _graphCrdt.Node.from({ updated: true });

              graph.merge((0, _defineProperty3.default)({}, node, node));
              _context44.next = 4;
              return db.commit(graph);

            case 4:
              cached = db.value(String(node));

              (0, _expect2.default)(cached).toExist();
              (0, _expect2.default)([].concat((0, _toConsumableArray3.default)(cached))).toEqual([].concat((0, _toConsumableArray3.default)(node)));

            case 7:
            case 'end':
              return _context44.stop();
          }
        }
      }, _callee44, undefined);
    })));

    it('triggers the after write pipeline', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee45() {
      var _afterWrite$calls$0$a, write;

      return _regenerator2.default.wrap(function _callee45$(_context45) {
        while (1) {
          switch (_context45.prev = _context45.next) {
            case 0:
              _context45.next = 2;
              return db.commit(graph);

            case 2:

              (0, _expect2.default)(afterWrite).toHaveBeenCalled();
              _afterWrite$calls$0$a = (0, _slicedToArray3.default)(afterWrite.calls[0].arguments, 1), write = _afterWrite$calls$0$a[0];


              (0, _expect2.default)(write.update).toBeA(_graphCrdt.Graph);
              (0, _expect2.default)(write.graph).toBeA(_graphCrdt.Graph);

            case 6:
            case 'end':
              return _context45.stop();
          }
        }
      }, _callee45, undefined);
    })));

    it('returns the deltas', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee46() {
      var result;
      return _regenerator2.default.wrap(function _callee46$(_context46) {
        while (1) {
          switch (_context46.prev = _context46.next) {
            case 0:
              node.merge({ commit: null });
              _context46.next = 3;
              return db.commit(graph);

            case 3:
              result = _context46.sent;


              (0, _expect2.default)(result).toBeAn(Object);
              (0, _expect2.default)(result.history).toBeA(_graphCrdt.Graph);
              (0, _expect2.default)(result.update).toBeA(_graphCrdt.Graph);

            case 7:
            case 'end':
              return _context46.stop();
          }
        }
      }, _callee46, undefined);
    })));

    it('transforms nodes into DB contexts', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee47() {
      var _node$meta, uid;

      return _regenerator2.default.wrap(function _callee47$(_context47) {
        while (1) {
          switch (_context47.prev = _context47.next) {
            case 0:
              node.merge({ count: 1 });
              _context47.next = 3;
              return db.commit(graph);

            case 3:
              _node$meta = node.meta(), uid = _node$meta.uid;

              (0, _expect2.default)(db.value(uid)).toBeA(_context54.default);

            case 5:
            case 'end':
              return _context47.stop();
          }
        }
      }, _callee47, undefined);
    })));

    it('turns node values into pointers', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee48() {
      var user;
      return _regenerator2.default.wrap(function _callee48$(_context48) {
        while (1) {
          switch (_context48.prev = _context48.next) {
            case 0:
              _context48.next = 2;
              return db.write('user', {});

            case 2:
              user = _context48.sent;
              _context48.next = 5;
              return db.write('users', (0, _defineProperty3.default)({}, user, user));

            case 5:

              (0, _expect2.default)(db.value('users').value(user)).toEqual({
                edge: String(user)
              });

            case 6:
            case 'end':
              return _context48.stop();
          }
        }
      }, _callee48, undefined);
    })));
  });

  describe('write()', function () {
    it('saves to the graph', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee49() {
      var node, name;
      return _regenerator2.default.wrap(function _callee49$(_context49) {
        while (1) {
          switch (_context49.prev = _context49.next) {
            case 0:
              _context49.next = 2;
              return db.write('user', { name: 'Jesse' });

            case 2:
              node = db.value('user');


              (0, _expect2.default)(node).toBeA(_context54.default);

              _context49.next = 6;
              return node.read('name');

            case 6:
              name = _context49.sent;

              (0, _expect2.default)(name).toBe('Jesse');

            case 8:
            case 'end':
              return _context49.stop();
          }
        }
      }, _callee49, undefined);
    })));

    it('does not overwrite, instead merges', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee50() {
      var user;
      return _regenerator2.default.wrap(function _callee50$(_context50) {
        while (1) {
          switch (_context50.prev = _context50.next) {
            case 0:
              _context50.next = 2;
              return db.write('user', { name: 'Bob' });

            case 2:
              _context50.next = 4;
              return db.write('user', { handle: 'BobTheGreat' });

            case 4:
              user = db.value('user');
              _context50.t0 = _expect2.default;
              _context50.next = 8;
              return user.read('name');

            case 8:
              _context50.t1 = _context50.sent;
              (0, _context50.t0)(_context50.t1).toBe('Bob');
              _context50.t2 = _expect2.default;
              _context50.next = 13;
              return user.read('handle');

            case 13:
              _context50.t3 = _context50.sent;
              (0, _context50.t2)(_context50.t3).toBe('BobTheGreat');

            case 15:
            case 'end':
              return _context50.stop();
          }
        }
      }, _callee50, undefined);
    })));

    it('updates node fields', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee51() {
      var user;
      return _regenerator2.default.wrap(function _callee51$(_context51) {
        while (1) {
          switch (_context51.prev = _context51.next) {
            case 0:
              _context51.next = 2;
              return db.write('user', { followers: 50 });

            case 2:
              user = _context51.sent;
              _context51.next = 5;
              return db.write('user', { followers: 49 });

            case 5:
              _context51.t0 = _expect2.default;
              _context51.next = 8;
              return user.read('followers');

            case 8:
              _context51.t1 = _context51.sent;
              (0, _context51.t0)(_context51.t1).toBe(49);
              _context51.next = 12;
              return db.write('user', { followers: 51 });

            case 12:
              _context51.t2 = _expect2.default;
              _context51.next = 15;
              return user.read('followers');

            case 15:
              _context51.t3 = _context51.sent;
              (0, _context51.t2)(_context51.t3).toBe(51);

            case 17:
            case 'end':
              return _context51.stop();
          }
        }
      }, _callee51, undefined);
    })));

    it('returns the written context', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee52() {
      var user, name;
      return _regenerator2.default.wrap(function _callee52$(_context52) {
        while (1) {
          switch (_context52.prev = _context52.next) {
            case 0:
              _context52.next = 2;
              return db.write('user', {
                name: 'Bob'
              });

            case 2:
              user = _context52.sent;


              (0, _expect2.default)(user).toBeA(_context54.default);
              (0, _expect2.default)(user).toBe(db.value('user'));

              _context52.next = 7;
              return user.read('name');

            case 7:
              name = _context52.sent;

              (0, _expect2.default)(name).toBe('Bob');

            case 9:
            case 'end':
              return _context52.stop();
          }
        }
      }, _callee52, undefined);
    })));
  });

  describe('API extensions', function () {
    var extend = function extend(api) {
      return (0, _root2.default)({
        extend: { root: api }
      });
    };

    it('are added to the root', function () {
      var db = extend({
        method: function method() {
          return true;
        },
        prop: 'yep'
      });

      (0, _expect2.default)(db.method).toBeA(Function);
      (0, _expect2.default)(db.prop).toBe('yep');
    });

    it('are non-enumerable', function () {
      var db = extend({ prop: 'value' });
      var keys = (0, _keys2.default)(db);
      (0, _expect2.default)(keys).toNotContain('prop');
    });

    it('are immutable', function () {
      var db = extend({ stuff: 'original' });
      (0, _expect2.default)(function () {
        db.stuff = 'something else';
      }).toThrow(Error);
    });
  });
});